@@ -1,13 +1,14 @@
 package Form::Tiny::Error;
 
-use v5.10; use warnings;
+use v5.10;
+use warnings;
 use Moo;
-use Types::Standard qw(Maybe Str Object ArrayRef InstanceOf);
+use Types::Standard qw(Maybe Str Object);
 use Carp qw(confess);
 
 use namespace::clean;
 
-our $VERSION = '1.00';
+our $VERSION = '1.99_01';
 
 use overload
 	q{""} => "as_string",
@@ -17,6 +18,7 @@ has "field" => (
 	is => "ro",
 	isa => Maybe [Str],
 	writer => "set_field",
+	default => sub { undef },
 );
 
 has "error" => (
@@ -92,17 +94,36 @@ __END__
 
 =head1 NAME
 
-Form::Tiny::Error - form validation process error
+Form::Tiny::Error - form error wrapper
 
 =head1 SYNOPSIS
 
+	my $error = Form::Tiny::Error::DoesNotValidate->new(
+		field => "some_field",
+		error => "some message"
+	);
+
 	my $field = $error->field; # field name or undef
-	my $error = $error->error; # error message or nested error object
+	my $data = $error->error; # error message or nested error object
 
 	my $message = $error->as_string;
 
 =head1 DESCRIPTION
 
-Form errors feature field name which caused validation error, error message and automatic stringification.
+The form errors class features field name which caused validation error, error message and automatic stringification.
 
 The C<< $error->error >> can return a nested error object in case of nested forms.
+
+A couple of in-place subclasses are provided to differentiate the type of error which occured. These are:
+
+=over
+
+=item * Form::Tiny::Error::InvalidFormat
+
+=item * Form::Tiny::Error::DoesNotExist
+
+=item * Form::Tiny::Error::IsntStrict
+
+=item * Form::Tiny::Error::DoesNotValidate
+
+=back
