@@ -1,20 +1,21 @@
 package Form::Tiny::FieldDefinition;
 
-use v5.10; use warnings;
+use v5.10;
+use warnings;
 use Moo;
-use Types::Standard qw(Enum Bool HasMethods CodeRef Maybe Str);
+use Types::Standard qw(Enum Bool HasMethods CodeRef Str InstanceOf);
 use Types::Common::String qw(NonEmptySimpleStr);
 use Carp qw(croak);
 use Scalar::Util qw(blessed);
 
+use Form::Tiny::Utils qw(try);
 use Form::Tiny::Error;
+use Form::Tiny::Path;
+use Form::Tiny::PathValue;
 
 use namespace::clean;
 
-our $VERSION = '1.00';
-
-our $nesting_separator = q{.};
-our $array_marker = q{*};
+our $VERSION = '1.99_01';
 
 has "name" => (
 	is => "ro",
@@ -22,6 +23,15 @@ has "name" => (
 	required => 1,
 );
 
+has "name_path" => (
+	is => "ro",
+	isa => InstanceOf ["Form::Tiny::Path"],
+	reader => "get_name_path",
+	init_arg => undef,
+	lazy => 1,
+	default => sub { Form::Tiny::Path->from_name(shift->name) },
+);
+
 has "required" => (
 	is => "ro",
 	isa => Enum [0, 1, "soft", "hard"],
@@ -47,6 +57,12 @@ has "adjust" => (
 	writer => "set_adjustment",
 );
 
+has "default" => (
+	is => "ro",
+	isa => CodeRef,
+	predicate => 1,
+);
+
 has "message" => (
 	is => "ro",
 	isa => Str,
@@ -67,11 +83,18 @@ sub BUILD
 
 		# checks for coercion == 1
 		my $t = $self->type;
-		croak "the type doesn't provide coercion"
+		croak "type doesn't provide coercion"
 			if !$self->has_type
 			|| !($t->can("coerce") && $t->can("has_coercion") && $t->has_coercion);
 	}
 
+	if ($self->has_default) {
+
+		croak "default value for an array field is unsupported"
+			if scalar grep { $_ eq 'ARRAY' } @{$self->get_name_path->meta};
+	}
+
+	# special case for subforms - set automatic adjustments
 	if ($self->is_subform && !$self->is_adjusted) {
 		$self->set_adjustment(sub { $self->type->fields });
 	}
@@ -84,15 +107,6 @@ sub is_subform
 	return $self->has_type && $self->type->DOES("Form::Tiny::Form");
 }
 
-sub get_name_path
-{
-	my ($self) = @_;
-
-	my $sep = quotemeta $nesting_separator;
-	my @parts = split /(?<!\\)$sep/, $self->name;
-	return map { s/\\$sep/$nesting_separator/g; $_ } @parts;
-}
-
 sub hard_required
 {
 	my ($self) = @_;
@@ -102,18 +116,32 @@ sub hard_required
 
 sub get_coerced
 {
-	my ($self, $value) = @_;
+	my ($self, $form, $value) = @_;
 
 	my $coerce = $self->coerce;
-	if (ref $coerce eq "CODE") {
-		return $coerce->($value);
-	}
-	elsif ($coerce) {
-		return $self->type->coerce($value);
-	}
-	else {
-		return $value;
+	my $coerced = $value;
+
+	my $error = try sub {
+		if (ref $coerce eq "CODE") {
+			$coerced = $coerce->($value);
+		}
+		elsif ($coerce) {
+			$coerced = $self->type->coerce($value);
+		}
+	};
+
+	if ($error) {
+		$form->add_error(
+			Form::Tiny::Error::DoesNotValidate->new(
+				{
+					field => $self->name,
+					error => $self->has_message ? $self->message : $error,
+				}
+			)
+		);
 	}
+
+	return $coerced;
 }
 
 sub get_adjusted
@@ -126,6 +154,25 @@ sub get_adjusted
 	return $value;
 }
 
+sub get_default
+{
+	my ($self, $form) = @_;
+
+	if ($self->has_default) {
+		my $default = $self->default->($form);
+		if (!$self->has_type || $self->type->check($default)) {
+			return Form::Tiny::PathValue->new(
+				path => $self->get_name_path->path,
+				value => $default,
+			);
+		}
+
+		croak 'invalid default value was set';
+	}
+
+	croak 'no default value set but was requested';
+}
+
 sub validate
 {
 	my ($self, $form, $value) = @_;
@@ -146,16 +193,18 @@ sub validate
 	}
 
 	if (!$valid) {
-		if ($self->is_subform && ref $error eq ref []) {
+		if ($self->is_subform && ref $error eq 'ARRAY') {
 			foreach my $exception (@$error) {
 				if (defined blessed $exception && $exception->isa("Form::Tiny::Error")) {
-					$exception->set_field(
-						join $nesting_separator,
-						$self->name, ($exception->field // ())
-					);
+					my $path = $self->get_name_path;
+					$path = $path->clone->append(HASH => $exception->field)
+						if defined $exception->field;
+
+					$exception->set_field($path->join);
 				}
 				else {
-					$exception = Form::Tiny::Error::DoesNotValidate->new({
+					$exception = Form::Tiny::Error::DoesNotValidate->new(
+						{
 							field => $self->name,
 							error => $exception,
 						}
@@ -166,7 +215,8 @@ sub validate
 			}
 		}
 		else {
-			my $exception = Form::Tiny::Error::DoesNotValidate->new({
+			my $exception = Form::Tiny::Error::DoesNotValidate->new(
+				{
 					field => $self->name,
 					error => $error,
 				}
@@ -185,7 +235,7 @@ __END__
 
 =head1 NAME
 
-Form::Tiny::FieldDefinition - a class representing field to be validated
+Form::Tiny::FieldDefinition - definition of a field to be validated
 
 =head1 SYNOPSIS
 
@@ -193,52 +243,57 @@ Form::Tiny::FieldDefinition - a class representing field to be validated
 	# name is the only required attribute
 	my $definition = Form::Tiny::FieldDefinition->new(
 		name => "something",
+		type => Str,
 		...
 	);
 
 =head1 DESCRIPTION
 
-Main class of the Form::Tiny system - this is a role that provides most of the module's functionality.
+This class keeps all the data for a field definition and contains method that handle single field validation.
 
 =head1 ATTRIBUTES
 
-Each of the attributes can be accessed by calling its name as a function on Form::Tiny::FieldDefinition object. See L<Form::Tiny::Manual> for more in depth examples.
+Each of the attributes can be accessed by calling its name as a function on Form::Tiny::FieldDefinition object. See L<Form::Tiny::Manual> for more examples.
 
 =head2 name
 
+The only required attribute for the constructor.
+
 A string which should specify the hash structure path of the field.
 
 Special characters are:
 
 =over
 
-=item * dot [.], which specifies nesting. Can be escaped with backslash [\]
+=item * dot [.], which specifies nesting
 
 =item * star [*], which specifies any number of array elements, but only if it is the only character on level, like a.*.b
 
 =back
 
+They both can be escaped by a backslash C<\> to lose their special meaning.
+
 =head2 required
 
-A field is not required by default (value 0), which means that its absence does not produce an error.
+A field is not required by default (value C<0>), which means that its absence does not produce an error.
 
-A field can also be soft required ("soft") or hard required ("hard" or 1).
+A field can also be soft required (C<"soft">) or hard required (C<"hard"> or C<1>).
 
-Soft required field errors only if it is undefined or not present in the input data.
+Soft required field produce errors only if it is undefined or not present in the input data.
 
 Hard required field also checks if the field is not an empty string.
 
 =head2 type
 
-A type is where you can plug in a Type::Tiny check. It has to be an instance of a class that provider I<validate> and I<check> methods, just like Type::Tiny. This can also be a different Form::Tiny form instance.
+The type attribute is where you can plug in a Type::Tiny type object. It has to be an instance of a class that provider I<validate> and I<check> methods, just like Type::Tiny. This can also be a Form::Tiny form instance.
 
 B<predicate:> I<has_type>
 
 =head2 coerce
 
-Coercions take place just before the validation. By default, values are not coerced. Specifying value I<1> will turn on coercions from the type object.
+Coercions take place just before the validation. By default, values are not coerced. Specifying value I<1> will cause the field to use coercions from the type object.
 
-It can also be a code reference which will be called to coerce the value.
+It can also be a code reference which will be called to coerce the value, passing in a field value as its only argument.
 
 =head2 adjust
 
@@ -248,6 +303,16 @@ B<predicate:> I<is_adjusted>
 
 B<writer:> I<set_adjustment>
 
+=head2 default
+
+A coderef returning the default value for the field. Will be used when the field is not present in the input at all. Making the field hard-required will make the default value be used in place of undefined / empty value as well.
+
+This coderef will be passed form instance as the only argument and is expected to return a scalar value.
+
+B<predicate>: I<has_default>
+
+B<writer>: I<set_default>
+
 =head2 message
 
 If type class error messages are not helpful enough, you can specify your own message string which will be inserted into form errors if the validation for the field fails.
@@ -266,24 +331,16 @@ B<predicate:> I<has_data>
 
 =head2 is_subform
 
-Checks if the field definition's type is a form - mixes in L<Form::Tiny::Form> role.
+Checks if the field definition's type is a form - whether it mixes in L<Form::Tiny::Form> role.
 
 =head2 get_name_path
 
-Parses and returns the name of the field as an array - a path to get the value in a hash.
+Parses and returns the name of the field as an object of L<Form::Tiny::Path> class.
 
 =head2 hard_required
 
 Checks if the field is hard-required (any of the two values which are allowed for this flag)
 
-=head2 get_coerced
-
-Coerces and returns a scalar value, according to the definition.
-
-=head2 get_adjusted
-
-Adjusts and returns a scalar value, according to the definition.
-
 =head2 validate
 
 Validates a scalar value. Arguments are C<$parent_form, $field_value>. Returns a boolean, whether the validation passed.
