@@ -1,13 +1,13 @@
 package Form::Tiny::Filter;
 
-use v5.10; use warnings;
+use v5.10;
+use warnings;
 use Moo;
 use Types::Standard qw(HasMethods CodeRef);
-use Carp qw(croak);
 
 use namespace::clean;
 
-our $VERSION = '1.00';
+our $VERSION = '1.99_01';
 
 has "type" => (
 	is => "ro",
@@ -19,17 +19,8 @@ has "code" => (
 	is => "ro",
 	isa => CodeRef,
 	required => 1,
-	writer => "set_code",
 );
 
-around "BUILDARGS" => sub {
-	my ($orig, $class, @args) = @_;
-
-	croak "Argument to Form::Tiny::Filter->new must be a single arrayref with two elements"
-		unless @args == 1 && ref $args[0] eq ref [] && @{$args[0]} == 2;
-	return {type => $args[0][0], code => $args[0][1]};
-};
-
 sub filter
 {
 	my ($self, $value) = @_;
@@ -47,19 +38,14 @@ __END__
 
 =head1 NAME
 
-Form::Tiny::Filter - a representation of filtering condition
+Form::Tiny::Filter - a representation of a filter
 
 =head1 SYNOPSIS
 
 	# in your form class
-	sub build_filters
-	{
-		return (
-			# the following will be coerced into Form::Tiny::Filter
-			# [ type, filtering sub ]
-			[Str, sub { ... }],
-		);
-	}
+
+	# the following will be coerced into Form::Tiny::Filter
+	form_filer Str, sub { uc shift() };
 
 =head1 DESCRIPTION
 
@@ -78,6 +64,7 @@ Required.
 A code reference accepting a single scalar and performing the filtering. The scalar will already be checked against the type.
 
 Required.
+
 =head1 METHODS
 
 =head2 filter
