@@ -1,16 +1,18 @@
 package Form::Tiny::PathValue;
 
-use v5.10; use warnings;
+use v5.10;
+use warnings;
 use Moo;
 use Types::Standard qw(ArrayRef);
 
 use namespace::clean;
 
-our $VERSION = '1.00';
+our $VERSION = '1.99_01';
 
 has "path" => (
 	is => "ro",
 	isa => ArrayRef,
+	required => 1,
 );
 
 has "value" => (
